function  [yc,iter,objc,ngrad] = GaussNewton(T,R,m,omega,alpha,beta,yc,yRef,hd,P,Dx,Dy)
maxIter = 500;
tolJ = 1e-3;
tolY = 1e-2;
tolG = 1e-2;

objc =  [];
ngrad = [];

Ts = getSplineCoefficients(T,'inter','splineInter','regularizer','moments','theta',1);
Rs = getSplineCoefficients(R,'inter','splineInter','regularizer','moments','theta',1);
R  = splineInterMex(Rs,omega,center(yRef,m));

func = @(yc) NPIRobjFctn(Ts,R,omega,m,yRef,yc,alpha,beta,P,hd,Dx,Dy);

iter = 1;
Jstop = func(yRef);
[Jc,dJ,H] = func(yc);
Jold  = Jc;
y0 = yc;
yOld = 0*yc;

objc =  [objc,Jc];
ngrad = [ngrad,norm(dJ)];
while 1
    % check stopping rules
    STOP(1) = (iter>0) && abs(Jold-Jc)  <= tolJ*(1+abs(Jstop));
    STOP(2) = (iter>0) && (norm(yc-yOld) <= tolY*(1+norm(y0)));
    STOP(3) = norm(dJ)      <= tolG*(1+abs(Jstop));
    STOP(4) = norm(dJ)      <= 1e6*eps;
    STOP(5) = (iter >= maxIter);
    if all(STOP(1:3)) || any(STOP(4:5))
        break;
    end

    iter = iter + 1;
    % solve the Gauss-Newton System
    dy = solveGN(-dJ,H);

    % check descent direction
    descent =   dJ' * dy;
    if descent > 0
        warning('no descent direction, switch to -dy!') 
        dy      = -dy;
    end

    % perform Armijo line-search
    yt = lineSearch(func,yc,dy,Jc,dJ,Dx,Dy);
    
    % save old values and update
    yOld = yc; Jold = Jc; yc = yt;
    [Jc,dJ,H] = func(yc);

    objc =  [objc,Jc];
    ngrad = [ngrad,norm(dJ)];
end 


end


function dy = solveGN(rhs,H)
maxIterCG = 100; tolCG = 1e-1;

D   = diag(H); % D is diagonal
PC = @(x) D.\x;
[dy,flag,relres,iter] = minres(H,rhs,tolCG,maxIterCG,PC);  
end