
function [Dx,Dy] = getDiscreteGradient(m,omega)

S  = speye(prod(m+1),prod(m+1));
PA = S(1:end-(m(1)+1),:); PA((m(1)+1)*(1:m(2)),:) = [];
PB = S(1:end-(m(1)+1),:); PB((m(1)+1)*(0:m(2)-1)+1,:) = [];
PC = S(m(1)+2:end,:);     PC((m(1)+1)*(1:m(2)),:) = [];
PD = S(m(1)+2:end,:);     PD((m(1)+1)*(0:m(2)-1)+1,:) = [];

PM = (1/4) * (PA + PB + PC + PD);

X = getNodalGrid(omega,m);
X = reshape(X,[],2);

X1 = X(:,1);
X2 = X(:,2);

[P1,Q1] = derivative(PA,PB,PM,X1,X2);
[P2,Q2] = derivative(PB,PD,PM,X1,X2);
[P3,Q3] = derivative(PD,PC,PM,X1,X2);
[P4,Q4] = derivative(PC,PA,PM,X1,X2);


Dx = [P1;P2;P3;P4];
Dy = [Q1;Q2;Q3;Q4];


function [Part1,Part2]=derivative(P1,P2,PM,X1,X2)

a = (P1-PM)*X1;
b = (P1-PM)*X2;
c = (P2-PM)*X1;
d = (P2-PM)*X2;

Det = a.*d-b.*c;
InvDet = sdiag(1./Det);

A = sdiag(a);
B = sdiag(b);
C = sdiag(c);
D = sdiag(d);

Part1 = InvDet*(D*(P1-PM)-B*(P2-PM));
Part2 = InvDet*(-C*(P1-PM)+A*(P2-PM));


function Result = sdiag(u)
n = length(u);
Result = spdiags(u,0,n,n);
