function [Sc,dS,d2S] = regularizer(uc,omega,m,hd,alpha,beta,Dx,Dy,varargin)

persistent A1 A2 A3 A4 A omegaOld mOld

if ~exist('mOld','var'),     mOld = [];     end
if ~exist('omegaOld','var'), omegaOld = []; end


dS =  [];
d2S = [];

for k=1:2:length(varargin) % overwrites default parameter
    eval([varargin{k},'=varargin{',int2str(k+1),'};']);
end

build = isempty(mOld) || isempty(omegaOld) ...
    || length(mOld) ~= length(m) || length(omegaOld) ~= length(omega) ...
    || any(mOld ~= m) || any(omegaOld~=omega);
if build
    mOld = m; omegaOld = omega;
    A1 = [Dx,-Dy];
    A2 = [Dy, Dx];
    A3 = [Dx, Dy];
    A4 = [-Dy,Dx];

    A = getGradientNodal(omega,m);
    A = (A'*A);
end

dS1  = alpha*hd*A*uc;
Sc1  = 0.5*uc'*dS1;

[Sc2,dS2,d2S2] = derivativeBeltrami(beta,uc,hd,A1,A2,A3,A4,doDerivative);
Sc = Sc1+Sc2;

if doDerivative
    dS = dS1+dS2;
    d2S = alpha*hd*A+d2S2;
end


function [d,dB,d2B]= derivativeBeltrami(alpha,uc,hd,A,B,C,D,doDerivative)
%%%%  penalty function is x/(x-1)^2

dB = [];
d2B = [];

Auc = A*uc;
Buc = B*uc;
Cuc = C*uc+2;
Duc = D*uc;

r1 = Auc.^2+Buc.^2;
r2 = 1./(Cuc.^2+Duc.^2);
r  = r1.*r2;
d = alpha*hd/4*sum(r./(r-1).^2);

if doDerivative
    dr1 = 2*sdiag(Auc)*A+2*sdiag(Buc)*B;
    r22 = r2.^2;
    dr2 = 2*sdiag(-r22.*Cuc)*C+2*sdiag(-r22.*Duc)*D;

    M  = sdiag(r1)*dr2+sdiag(r2)*dr1;
    MT = M';

    d1 = (-r-1)./(r-1).^3;
    dB = alpha*hd/4*M'*d1;

    d2 = (2*r+4)./(r-1).^4;
    d2B = alpha*hd/4*(MT*sdiag(d2)*M);
end

function Result = sdiag(u)
Result = spdiags(u,0,length(u),length(u));